-- Flowmaster
-- By Matthew "Kiminari of Icecrown" Del Buono
--
-- Copyright (c) 2011 Matthew P. Del Buono
-- All Rights Reserved
--
-- You may not redistribute this software, in whole or in part,
-- verbatim or modified, without the express, written consent of the author.

local _, Addon = ...;

-- SV data, stores all gold info
FlowMasterData = {
	--[[ Structure:
	
	[guild] = {
		mine = aaa,
		lastMine = ddd or 0 if never checked,
		lastOther = bbb or 0 if never checked,
		lastTotal = ccc or 0 if never checked,
		lastCheck = timestamp or 0 if never checked
	};
	]]
};


-- Event dispatcher
local dispatchFrame = CreateFrame("FRAME");
dispatchFrame:SetScript("OnEvent", function(self, e, ...)
	Addon[e](...)
end);

-- Handler for logging in (sets up the guild info)
-- Since PLAYER_ENTERING_WORLD is too early, we can use
-- GUILD_ROSTER_UPDATE which fires at each login. Running this multiple
-- times is not a problem anyway.
local guild;
function Addon.GUILD_ROSTER_UPDATE(...)
	guild = GetGuildInfo("player");
	if guild ~= nil and #guild > 0 then
		-- Initialize guild data
		FlowMasterData[guild] = FlowMasterData[guild] or {
			mine = 0,
			lastMine = 0,
			lastOther = 0,
			lastTotal = 0,
			lastCheck = 0
		};
	end
end
dispatchFrame:RegisterEvent("GUILD_ROSTER_UPDATE");


-- Handler for opening the guild bank. Records all data about other users
-- and pumps the timestamp
do
	local function DoInitialPopulation()
		FlowMasterData[guild].lastTotal = GetGuildBankMoney();
		FlowMasterData[guild].lastCheck = time();
		FlowMasterData[guild].lastMine = FlowMasterData[guild].mine;
	end

	dispatchFrame:RegisterEvent("GUILDBANKLOG_UPDATE");
	function Addon.GUILDBANKLOG_UPDATE()
		local guildData = FlowMasterData[guild];
		local lastCheck = guildData.lastCheck;
		if lastCheck == 0 then
			return DoInitialPopulation();
		end
		
		-- Do a normal check
		local timeSinceLastCheck = time() - lastCheck;

		if timeSinceLastCheck < 60 * 60 then	-- We only have hourly precision on the guild bank. This is the best we can do
			return print("Warning: Flowmaster not updating guild member contributions because it was updated less than an hour ago.");
		end

		-- Add up all of the transactions since last check
		local total = 0;
		for i = 1, GetNumGuildBankMoneyTransactions() do
			local type, name, amount, year, month, day, hour = GetGuildBankMoneyTransaction(i);
			local timeSinceTransaction = 
				hour * 60 * 60 +
				day * 24 * 60 * 60 +
				month * 31 * 24 * 60 * 60 + -- best we can do, unfortunately this is not consistent and it's all we get
				year * 365 * 24 * 60 * 60; -- Same here


			if timeSinceTransaction < timeSinceLastCheck then
				-- Newer than last check; record it
				local multiplier;
				if type == "deposit" then
					multiplier = 1;
				elseif type == "repair" or type == "withdraw" then
					multiplier = -1;
				else
					error("Unknown guild log type '" .. type .. ". Please report this error.");
				end

				total = total + multiplier * amount;
				print("Detected transaction '"..type.."' by '"..name.."' for " .. (multiplier * amount));
			end

			
		end

		-- Take the difference between log and totals to compute others' contributions
		print(GetGuildBankMoney() - guildData.lastTotal);
		print(total);
		print(guildData.mine - guildData.lastMine);
		local difference = GetGuildBankMoney() - guildData.lastTotal - total - (guildData.mine - guildData.lastMine);
		assert(difference >= 0);
		guildData.lastOther = guildData.lastOther + difference;
		
		-- Update everything else
		guildData.lastMine = guildData.mine;
		guildData.lastTotal = GetGuildBankMoney();
		guildData.lastCheck = time();
	end
end


-- Handler for looting money. Records it as money earned for the guild.
do
	local function ParseGuildMoney(msg)
		local amount = msg:match("You loot [0-9a-zA-Z%s,]+ %(([0-9a-zA-Z%s,]+) deposited to guild bank%)");
		if not amount then
			amount = msg:match("Your share of the loot is [0-9a-zA-Z%s,]+ %(([0-9a-zA-Z%s,]+) deposited to guild bank%)");
		end

		if amount == nil then
			-- No parse, probably because don't have the cash flow perk
			return 0;
		end

		-- Pick out the right one
		local gold, silver, copper = amount:match("([0-9]+) Gold, ([0-9]+) Silver, ([0-9]+) Copper");
		if gold then
			return tonumber(gold) * 10000 + tonumber(silver) * 100 + tonumber(copper)
		end

		local silver, copper = amount:match("([0-9]+) Silver, ([0-9]+) Copper");
		if silver then
			return tonumber(silver) * 100 + tonumber(copper);
		end

		local copper = tonumber(amount:match("([0-9]+) Copper"));
		assert(copper);
		return copper;
	end

	function Addon.CHAT_MSG_MONEY(msg)
		local money = ParseGuildMoney(msg);
		FlowMasterData[guild].mine = FlowMasterData[guild].mine + money;
	end
	dispatchFrame:RegisterEvent("CHAT_MSG_MONEY");
end


-- Slash command handler
SLASH_FLOWMASTER1 = "/flowmaster";
SLASH_FLOWMASTER2 = "/fm";
do
	-- Formats the copper amount into a reasonable output format
	local function FormatOutput(copper)
		if copper < 100 then
			return copper .. " Copper";
		elseif copper < 10000 then
			return math.floor(copper / 100) .. " Silver, " .. copper % 100 .. " Copper";
		else
			return math.floor(copper / 10000) .. " Gold, " .. math.floor((copper % 10000) / 100) .. " Silver, " .. copper % 100 .. " Copper";
		end
	end

	SlashCmdList["FLOWMASTER"] = function()

		print("Your guild contributions through loot: " .. FormatOutput(FlowMasterData[guild].mine));
		print("Other members' contributions (estimated): " .. FormatOutput(FlowMasterData[guild].lastOther));
		print("Other members last updated: " .. date("%c", FlowMasterData[guild].lastCheck));
	end
end
